package tjm.tdg;

import java.io.File;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;

import tjm.tdg.ResourceManager.MODEL;
import tjm.tdg.algorithms.PNG;
import tjm.tdg.gui.GuiScreen;
import tjm.tdg.gui.GuiScreenGameOver;
import tjm.tdg.gui.GuiScreenInGame;
import tjm.tdg.gui.GuiScreenMain;
import tjm.tdg.gui.GuiScreenPause;
import android.opengl.GLSurfaceView;
import android.view.MotionEvent;

import com.threed.jpct.Camera;
import com.threed.jpct.FrameBuffer;
import com.threed.jpct.Light;
import com.threed.jpct.Object3D;
import com.threed.jpct.RGBColor;
import com.threed.jpct.SimpleVector;
import com.threed.jpct.World;
import com.threed.jpct.util.MemoryHelper;

public class Renderer implements GLSurfaceView.Renderer {
	// 32 ms (30 FPS, closest 2^n => 32; bypass for floating point errors)
	private final Ticker ticker = new Ticker(32);
	public GameWorld gameWorld = null;
	public boolean takeScreen = false;

	private final RGBColor back = new RGBColor(50, 50, 100);
	protected FrameBuffer fb = null;
	public World world;
	protected Light sun;
	protected int width = 0, height = 0;

	public GuiScreen currentGui = null;
	public GuiScreen guiMain;
	public GuiScreen guiInGame;
	public GuiScreen guiPause;
	public GuiScreen guiGameOver;

	private long lastTime = 0;
	private int fpsCounter = 0;
	public static int fps = 0;

	public void createNewWorld() {
		this.gameWorld = new GameWorld(this, Settings.LEVEL_WIDTH, Settings.LEVEL_HEIGHT);

		createGui();

		this.world = new World();
		this.world.setAmbientLight(75, 75, 75);

		Object3D floor = ResourceManager.getModelCopy(MODEL.FLOOR);
		floor.translate(Settings.CAM_OFFSET_X, 0, Settings.CAM_OFFSET_Z);

		this.world.addObject(floor);

		this.sun = new Light(this.world);
		this.sun.setIntensity(250, 250, 250);
		this.sun.setPosition(new SimpleVector(0, -50, 0));

		Camera cam = this.world.getCamera();
		cam.rotateCameraZ((float) (Math.PI / 2));

		setCameraPosition(Settings.CAM_OFFSET_X, Settings.CAM_OFFSET_Y, Settings.CAM_OFFSET_Z - 10);
	}

	public void createGui() {
		this.guiMain = new GuiScreenMain(this.gameWorld, this.fb, this.width, this.height);
		this.guiInGame = new GuiScreenInGame(this.gameWorld, this.fb, this.width, this.height);
		this.guiPause = new GuiScreenPause(this.gameWorld, this.fb, this.width, this.height);
		this.guiGameOver = new GuiScreenGameOver(this.gameWorld, this.fb, this.width, this.height);
	}

	public void setCameraPosition(float x, float y, float z) {
		Camera cam = this.world.getCamera();

		cam.setPosition(x, y, z);
		cam.lookAt(new SimpleVector(Settings.CAM_OFFSET_X, 0, Settings.CAM_OFFSET_Z));
	}

	@Override
	public void onSurfaceCreated(GL10 gl, EGLConfig config) {
		createNewWorld();
	}

	@Override
	public void onSurfaceChanged(GL10 gl, int w, int h) {
		try {
			if (this.fb != null) {
				this.fb.dispose();
			}
			this.fb = new FrameBuffer(w, h);

			this.width = w;
			this.height = h;

			createGui();

			MemoryHelper.compact();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	@Override
	public void onDrawFrame(GL10 gl) {
		try {
			int ticks = 0;

			this.fpsCounter++;

			long nowTime = Ticker.getTime();
			if (nowTime - this.lastTime > 1000) {
				fps = (int) (this.fpsCounter / ((nowTime - this.lastTime) / 1000.0));
				this.lastTime = nowTime;
				this.fpsCounter = 0;
			}

			ticks = this.ticker.getTicks();

			this.fb.clear(this.back);

			if (this.currentGui == null) {
				this.currentGui = this.guiMain;
			}

			if (this.currentGui == this.guiInGame && this.gameWorld.isGameOver()) {
				this.currentGui = this.guiGameOver;
			}

			if (this.currentGui == this.guiInGame || this.currentGui == this.guiPause || this.currentGui == this.guiGameOver) {
				if (this.currentGui == this.guiInGame && ticks > 0) {
					try {
						this.gameWorld.update(ticks);
					} catch (Exception e) {
						e.printStackTrace();
					}
				}

				this.world.renderScene(this.fb);
				this.world.draw(this.fb);

				this.currentGui.draw();
			} else if (this.currentGui != null) {
				this.currentGui.draw();
			}

			this.fb.display();

			if (this.takeScreen) {
				this.takeScreen = false;
				saveScreen();
				this.ticker.getTicks();
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void onTouchEvent(MotionEvent me) {
		if (me.getAction() == MotionEvent.ACTION_DOWN) {
			this.currentGui.onClick((int) me.getX(), (int) me.getY());
		} else if (this.currentGui instanceof GuiScreenInGame) {
			this.gameWorld.onTouchEvent(me);
		}
	}

	private void saveScreen() {
		try {
			File fileLocation = new File(ResourceManager.LOCATION_IMG, "/tdg-" + Ticker.getTime() + ".png");
			if (!fileLocation.exists()) {
				if (!fileLocation.createNewFile()) {
					return;
				}
			}

			int[] tmpPixels = this.fb.getPixels();
			byte[] data = new byte[tmpPixels.length * 4];
			for (int i = 0; i < tmpPixels.length; i++) {
				// RGBA
				data[i * 4 + 0] = (byte) ((tmpPixels[i] & 0x000000ff) >>> 0);
				data[i * 4 + 1] = (byte) ((tmpPixels[i] & 0x0000ff00) >>> 8);
				data[i * 4 + 2] = (byte) ((tmpPixels[i] & 0x00ff0000) >>> 16);
				data[i * 4 + 3] = (byte) ((tmpPixels[i] & 0xff000000) >>> 24);
			}

			PNG.write(fileLocation, data, 0, this.fb.getWidth(), this.fb.getHeight(), 4);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
