package tjm.tdg;

import java.io.File;
import java.util.HashMap;
import java.util.Map;
import java.util.Random;

import raft.glfont.android.AGLFont;
import raft.glfont.android.TexturePack;
import android.content.Context;
import android.graphics.Paint;
import android.graphics.Typeface;
import android.media.AudioManager;
import android.media.SoundPool;
import android.media.SoundPool.OnLoadCompleteListener;
import android.os.Environment;
import android.util.Log;

import com.threed.jpct.FrameBuffer;
import com.threed.jpct.Loader;
import com.threed.jpct.Matrix;
import com.threed.jpct.Object3D;
import com.threed.jpct.SimpleVector;
import com.threed.jpct.Texture;
import com.threed.jpct.TextureManager;
import com.threed.jpct.util.BitmapHelper;

public class ResourceManager {
	// texture constants
	public final static class TEXTURE {
		public final static String BERSERKER = "tex_berserker";
		public final static String KAMIKAZE = "tex_kamikaze";
		public final static String TACTICAL = "tex_tactical";

		public final static String PROJ_ALLY = "tex_proj_ally";
		public final static String PROJ_ENEMY = "tex_proj_enemy";

		public final static String TOWER_BASE = "tex_tower_base";
		public final static String TOWER_BASIC = "tex_tower_basic";
		public final static String TOWER_LONG_RANGE = "tex_tower_long_range";
		public final static String TOWER_SPLASH = "tex_tower_splash";

		public final static String FLOOR = "tex_floor";
	}

	// image constants
	public final static class IMAGE {
		public final static String TOWER01 = "img_tower01";
		public final static String TOWER02 = "img_tower02";
		public final static String TOWER03 = "img_tower03";

		public final static String PLAY = "img_play";
		public final static String PAUSE = "img_pause";
		public final static String BUTTON = "img_button";

		public final static String BACKGROUND_MAIN = "img_bg_main";
	}

	// sound constants
	public final static class SOUND {
		public final static String ATTACK01 = "snd_attack01";
		public final static String ATTACK02 = "snd_attack02";
		public final static String ATTACK03 = "snd_attack03";
		public final static String BACKGROUND = "snd_background";
	}

	// models
	public final static class MODEL {
		public final static String BERSERKER = "mod_berserker";
		public final static String KAMIKAZE = "mod_kamikaze";
		public final static String TACTICAL = "mod_tactical";

		public final static String PROJ_ALLY = "mod_proj_ally";
		public final static String PROJ_ENEMY = "mod_proj_enemy";

		public final static String TOWER_BASE = "mod_tower_base";
		public final static String TOWER_BASIC = "mod_tower_basic";
		public final static String TOWER_LONG_RANGE = "mod_tower_long_range";
		public final static String TOWER_SPLASH = "mod_tower_splash";

		public final static String FLOOR = "mod_floor";
	}

	public static AGLFont FONT1;
	public static AGLFont FONT2;
	public static AGLFont FONT3;

	// sd card location
	public final static File LOCATION_SD_CARD = Environment.getExternalStorageDirectory();
	public final static File LOCATION_IMG = new File(LOCATION_SD_CARD, "/img");

	public static int lastSound = 0;

	public final static TextureManager texManager = TextureManager.getInstance();
	public final static TexturePack texturePack = new TexturePack();
	public static Random rand = new Random();
	public static Context context;
	private static AudioManager audioManager;
	private static SoundPool soundPool;
	private final static Map<String, Integer> SOUNDS = new HashMap<String, Integer>();
	private final static Map<String, Object3D> MODELS = new HashMap<String, Object3D>();
	private final static Map<String, Integer> IMAGES = new HashMap<String, Integer>();

	public static void init(Context ctx) {
		context = ctx;

		audioManager = (AudioManager) context.getSystemService(Context.AUDIO_SERVICE);

		soundPool = new SoundPool(20, AudioManager.STREAM_MUSIC, 0);
		soundPool.setOnLoadCompleteListener(new OnLoadCompleteListener() {
			@Override
			public void onLoadComplete(SoundPool sPool, int id, int status) {
				Log.i("LOG", "loaded sound: " + id + "," + status);
			}
		});

		if (LOCATION_SD_CARD.exists()) {
			if (!LOCATION_IMG.exists()) {
				if (!LOCATION_IMG.mkdirs()) {
					Log.i("LOG", "mkdirs() failed");
				}
			}
		}

		Paint paint = new Paint();
		paint.setAntiAlias(true);
		paint.setTypeface(Typeface.create((String) null, Typeface.BOLD));
		paint.setTextSize(16);
		FONT1 = new AGLFont(paint);

		paint.setAntiAlias(true);
		paint.setTextSize(20);
		FONT2 = new AGLFont(paint);

		paint.setAntiAlias(true);
		paint.setTextSize(32);
		FONT3 = new AGLFont(paint);
	}

	@Deprecated
	public static void loadModel(String name, Object3D model, String texture) {
		model.calcTextureWrapSpherical();
		model.setTexture(texture);
		model.strip();
		model.build();
		MODELS.put(name, model);
	}

	public static void loadModel(String name, int resourceId, float scale, String texture) {
		if (!MODELS.containsKey(name)) {
			Object3D[] models = Loader.load3DS(context.getResources().openRawResource(resourceId), scale);
			Object3D model = new Object3D(0);
			Object3D temp = null;

			for (int i = 0; i < models.length; i++) {
				temp = models[i];
				temp.setCenter(SimpleVector.ORIGIN);
				temp.rotateX((float) (-0.5 * Math.PI));
				temp.rotateMesh();
				temp.setRotationMatrix(new Matrix());
				model = Object3D.mergeObjects(model, temp);
				model.build();
			}

			model.setTexture(texture);

			MODELS.put(name, model);
		}
	}

	public static Object3D getModelCopy(String name) {
		if (MODELS.containsKey(name)) {
			return MODELS.get(name).cloneObject();
		}
		return null;
	}

	public static void loadTexture(String name, int resourceId, int width, int height) {
		if (!texManager.containsTexture(name)) {
			texManager.addTexture(name, new Texture(BitmapHelper.rescale(BitmapHelper.convert(context.getResources().getDrawable(resourceId)), width, height)));
		}
	}

	public static void loadImage(String name, int resourceId) {
		if (!IMAGES.containsKey(name)) {
			IMAGES.put(name, texturePack.addImage(BitmapHelper.convert(context.getResources().getDrawable(resourceId))));
		}
	}

	public static void packImages() {
		texturePack.pack(true);
	}

	public static void blitImage(FrameBuffer fb, String image, int x, int y, int width, int height) {
		if (IMAGES.containsKey(image)) {
			texturePack.blit(fb, IMAGES.get(image), x, y, width, height, 0xFF, false, null);
		}
	}

	public static void loadSound(String name, int resourceId) {
		if (!SOUNDS.containsKey(name)) {
			SOUNDS.put(name, soundPool.load(context, resourceId, 1));
		}
	}

	public static int playSound(String name, float localVolume) {
		return playSound(name, 0, localVolume * Settings.volEffect);
	}

	public static int loopSound(String name, int streamID, float localVolume) {
		if (streamID != 0) {
			stopSound(streamID);
		}

		return playSound(name, -1, localVolume * Settings.volBackground);
	}

	private static int playSound(String name, int loop, float localVolume) {
		float actualVolume = audioManager.getStreamVolume(AudioManager.STREAM_MUSIC);
		float maxVolume = audioManager.getStreamMaxVolume(AudioManager.STREAM_MUSIC);
		float volume = (actualVolume / maxVolume) * localVolume;

		if (SOUNDS.containsKey(name)) {
			return soundPool.play(SOUNDS.get(name), volume, volume, 1, loop, 1f);
		}

		return 0;
	}

	public static void stopSound(int streamID) {
		soundPool.stop(streamID);
	}

	public static void pauseAllSound() {
		soundPool.autoPause();
	}

	public static void resumeAllSound() {
		soundPool.autoResume();
	}

	public static void drawString(String string, int font, int x, int y, double r, double g, double b) {
		// TODO: implement
	}
}
