package tjm.tdg;

import tjm.tdg.ResourceManager.IMAGE;
import tjm.tdg.ResourceManager.MODEL;
import tjm.tdg.ResourceManager.SOUND;
import tjm.tdg.ResourceManager.TEXTURE;
import android.app.Activity;
import android.content.Intent;
import android.media.AudioManager;
import android.opengl.GLSurfaceView;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.view.MotionEvent;

import com.threed.jpct.Primitives;

public class TDG extends Activity {
	private GLSurfaceView mGLView = null;
	public static Renderer renderer = null;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		try {
			this.mGLView = new GLSurfaceView(getApplication());

			// Enable the OpenGL ES2.0 context (doesn't work on the emulator)
			this.mGLView.setEGLContextClientVersion(2);

			if (renderer == null) {
				renderer = new Renderer();
			}
			this.mGLView.setRenderer(renderer);
			setContentView(this.mGLView);

			ResourceManager.init(this);

			// load textures
			ResourceManager.loadTexture(TEXTURE.BERSERKER, R.raw.mob01_skin, 16, 16);
			ResourceManager.loadTexture(TEXTURE.KAMIKAZE, R.raw.mob02_skin, 16, 16);
			ResourceManager.loadTexture(TEXTURE.TACTICAL, R.raw.texture05, 16, 16);

			ResourceManager.loadTexture(TEXTURE.PROJ_ALLY, R.raw.projectile_skin_ally, 16, 16);
			ResourceManager.loadTexture(TEXTURE.PROJ_ENEMY, R.raw.projectile_skin_enemy, 16, 16);

			ResourceManager.loadTexture(TEXTURE.TOWER_BASE, R.raw.tower_base_skin, 128, 128);
			ResourceManager.loadTexture(TEXTURE.TOWER_BASIC, R.raw.tower01_skin, 128, 128);
			ResourceManager.loadTexture(TEXTURE.TOWER_LONG_RANGE, R.raw.tower02_skin, 128, 128);
			ResourceManager.loadTexture(TEXTURE.TOWER_SPLASH, R.raw.texture04, 128, 128);

			ResourceManager.loadTexture(TEXTURE.FLOOR, R.raw.floor_skin, 128, 128);

			// load sounds
			ResourceManager.loadSound(SOUND.ATTACK01, R.raw.attack01);
			ResourceManager.loadSound(SOUND.ATTACK02, R.raw.attack02);
			ResourceManager.loadSound(SOUND.ATTACK03, R.raw.attack03);
			ResourceManager.loadSound(SOUND.BACKGROUND, R.raw.background05);

			// load models
			ResourceManager.loadModel(MODEL.BERSERKER, R.raw.mob01_model, 1.0f / 32.0f, TEXTURE.BERSERKER);
			ResourceManager.loadModel(MODEL.KAMIKAZE, R.raw.mob02_model, 1.0f / 32.0f, TEXTURE.KAMIKAZE);
			ResourceManager.loadModel(MODEL.TACTICAL, R.raw.mob03_model, 1.0f / 32.0f, TEXTURE.TACTICAL);

			ResourceManager.loadModel(MODEL.PROJ_ALLY, Primitives.getSphere(0.15f), TEXTURE.PROJ_ALLY);
			ResourceManager.loadModel(MODEL.PROJ_ENEMY, Primitives.getSphere(0.15f), TEXTURE.PROJ_ENEMY);

			ResourceManager.loadModel(MODEL.TOWER_BASE, R.raw.tower_base_model, 1.0f / 16.0f, TEXTURE.TOWER_BASE);
			ResourceManager.loadModel(MODEL.TOWER_BASIC, R.raw.tower01_model, 1.0f / 32.0f, TEXTURE.TOWER_BASIC);
			ResourceManager.loadModel(MODEL.TOWER_LONG_RANGE, R.raw.tower02_model, 1.0f / 32.0f, TEXTURE.TOWER_LONG_RANGE);
			ResourceManager.loadModel(MODEL.TOWER_SPLASH, R.raw.tower03_model, 1.0f / 32.0f, TEXTURE.TOWER_SPLASH);

			ResourceManager.loadModel(MODEL.FLOOR, R.raw.floor_3ds, 1.0f, TEXTURE.FLOOR);

			ResourceManager.loadImage(IMAGE.TOWER01, R.raw.tower01);
			ResourceManager.loadImage(IMAGE.TOWER02, R.raw.tower02);
			ResourceManager.loadImage(IMAGE.TOWER03, R.raw.tower03);
			ResourceManager.loadImage(IMAGE.PLAY, R.raw.play);
			ResourceManager.loadImage(IMAGE.PAUSE, R.raw.pause);
			ResourceManager.loadImage(IMAGE.BUTTON, R.raw.button);
			ResourceManager.loadImage(IMAGE.BACKGROUND_MAIN, R.raw.background_main);
			ResourceManager.packImages();

			// keep the screen on
			getWindow().addFlags(android.view.WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);

			setVolumeControlStream(AudioManager.STREAM_MUSIC);

			ResourceManager.lastSound = ResourceManager.loopSound(SOUND.BACKGROUND, ResourceManager.lastSound, 0.5f);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.activity_tdg, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case R.id.menu_settings:
			Intent launchNewIntent = new Intent(this, SettingsActivity.class);
			startActivityForResult(launchNewIntent, 0);
			return true;
		case R.id.menu_screenshot:
			renderer.takeScreen = true;
			return true;
		case R.id.menu_exit:
			ResourceManager.stopSound(ResourceManager.lastSound);
			finish();
		default:
			return super.onOptionsItemSelected(item);
		}
	}

	@Override
	protected void onPause() {
		super.onPause();

		ResourceManager.pauseAllSound();
	}

	@Override
	protected void onResume() {
		super.onResume();

		ResourceManager.resumeAllSound();
	}

	@Override
	protected void onStop() {
		super.onStop();
	}

	@Override
	public boolean onTouchEvent(MotionEvent me) {
		renderer.onTouchEvent(me);

		return true;
	}
}
