package tjm.tdg.entities.towers;

import tjm.tdg.GameWorld;
import tjm.tdg.ResourceManager;
import tjm.tdg.entities.Entity;
import tjm.tdg.entities.Team;
import tjm.tdg.entities.projectiles.Projectile;
import tjm.tdg.entities.projectiles.ProjectileAlly;
import tjm.tdg.entities.projectiles.ProjectileEnemy;
import tjm.tdg.utils.Vector2f;

public abstract class Tower extends Entity {
	public int attackSpeed = 0;
	public int attackCooldown = 0;
	public double attackRange = 0;

	public Tower(GameWorld gameWorld, Vector2f pos) {
		this.world = gameWorld;
		this.position = pos.clone();
		this.positionPrev = pos.clone();
		this.radius = new Vector2f(0.5f, 0.5f);
		this.isBlocking = true;
		this.isImmortal = false;
		this.attackCooldown = 0;
		this.team = Team.ALLY;

		this.modelOffsetY = -1.0f;
	}

	@Override
	public boolean isRemoved() {
		return this.health <= 0;
	}

	@Override
	public boolean isInRange(Entity entity) {
		return super.isInRange(entity, this.attackRange);
	}

	@Override
	public boolean canAttack() {
		return this.attackCooldown <= 0;
	}

	@Override
	public Projectile attack(Entity entity) {
		super.attack(entity);

		Projectile projectile = null;

		this.attackCooldown = this.attackSpeed;

		if (this.team == Team.ALLY) {
			projectile = new ProjectileAlly(this.world, this.position, this, entity);
		} else {
			projectile = new ProjectileEnemy(this.world, this.position, this, entity);
		}

		projectile.attackDamage = (int) (this.attackDamage * (ResourceManager.rand.nextDouble() / 2 + 0.75));

		return projectile;
	}

	@Override
	public void update(int ticks) {
		super.update(ticks);

		this.attackCooldown -= ticks;
	}
}
