package tjm.tdg.gui;

import tjm.tdg.GameWorld;
import tjm.tdg.ResourceManager;
import tjm.tdg.ResourceManager.IMAGE;
import tjm.tdg.ResourceManager.MODEL;
import tjm.tdg.Settings;
import tjm.tdg.TDG;
import tjm.tdg.entities.EntityType;

import com.threed.jpct.FrameBuffer;
import com.threed.jpct.RGBColor;

public class GuiScreenInGame extends GuiScreen {
	private final GuiControlButton btnTower1;
	private final GuiControlButton btnTower2;
	private final GuiControlButton btnTower3;
	private final GuiControlButtonCenter btnPause;

	public GuiScreenInGame(GameWorld world, FrameBuffer frameBuffer, int width, int height) {
		super(world, frameBuffer, width, height);

		int id = 0;
		this.controls.add(this.btnTower1 = new GuiControlButton(id++, this, width / 2 - 96, height - 64, 64, 64, "@" + Settings.TOWER_COST_BASIC, IMAGE.TOWER01));
		this.controls.add(this.btnTower2 = new GuiControlButton(id++, this, width / 2 - 16, height - 64, 64, 64, "@" + Settings.TOWER_COST_LONG_RANGE, IMAGE.TOWER02));
		this.controls.add(this.btnTower3 = new GuiControlButton(id++, this, width / 2 + 64, height - 64, 64, 64, "@" + Settings.TOWER_COST_SPLASH, IMAGE.TOWER03));
		this.controls.add(this.btnPause = new GuiControlButtonCenter(id++, this, width - 64, height - 64, 64, 64, "", IMAGE.PAUSE));
	}

	@Override
	public void onClick(GuiControl guiControl) {
		if (guiControl.id == this.btnTower1.id && this.gameWorld.coins >= Settings.TOWER_COST_BASIC) {
			this.gameWorld.touchAction = 1;
			this.gameWorld.placingTower = true;
			this.gameWorld.placingTowerModel = ResourceManager.getModelCopy(MODEL.TOWER_BASIC);
			this.gameWorld.placingTowerType = EntityType.TOWER_BASIC;
		} else if (guiControl.id == this.btnTower2.id && this.gameWorld.coins >= Settings.TOWER_COST_LONG_RANGE) {
			this.gameWorld.touchAction = 1;
			this.gameWorld.placingTower = true;
			this.gameWorld.placingTowerModel = ResourceManager.getModelCopy(MODEL.TOWER_LONG_RANGE);
			this.gameWorld.placingTowerType = EntityType.TOWER_LONG_RANGE;
		} else if (guiControl.id == this.btnTower3.id && this.gameWorld.coins >= Settings.TOWER_COST_SPLASH) {
			this.gameWorld.touchAction = 1;
			this.gameWorld.placingTower = true;
			this.gameWorld.placingTowerModel = ResourceManager.getModelCopy(MODEL.TOWER_SPLASH);
			this.gameWorld.placingTowerType = EntityType.TOWER_SPLASH;
		} else if (guiControl.id == this.btnPause.id) {
			TDG.renderer.currentGui = TDG.renderer.guiPause;
			ResourceManager.pauseAllSound();
		}

		if (this.gameWorld.placingTower) {
			this.gameWorld.renderer.world.addObject(this.gameWorld.placingTowerModel);
			this.gameWorld.placingTowerModel.translate(-100, 0, 0);
		}
	}

	@Override
	public void draw() {
		super.draw();

		drawString(this.fb, ResourceManager.FONT1, "Coins: @" + this.gameWorld.coins, 10, this.height - 10 - 16 - 16, RGBColor.WHITE);
		drawString(this.fb, ResourceManager.FONT1, "Units killed: " + this.gameWorld.deadEnemy, 10, this.height - 10 - 16, RGBColor.WHITE);
		drawString(this.fb, ResourceManager.FONT1, "Units lost: " + this.gameWorld.deadAlly, 10, this.height - 10, RGBColor.WHITE);

		drawStringCenter(this.fb, ResourceManager.FONT1, this.gameWorld.status(), this.width / 2, 16, RGBColor.WHITE);
	}
}
